@@ -22,12 +22,14 @@
 #define _GENGETOPT_H
 
 #include "acceptedvalues.h"
+#include "ggos.h"
 
 int gengetopt_define_package (char * s) ;
 int gengetopt_define_version (char * s) ;
 int gengetopt_define_purpose (char * s) ;
 void gengetopt_set_section (const char * s, const char *sd) ;
 int gengetopt_add_group (const char * s, const char *desc, int required) ;
+int gengetopt_has_option (const char * long_opt, char short_opt);
 int gengetopt_add_option (const char * long_opt, char short_opt,
                           const char * desc,
                           int type, int flagstat, int required,
@@ -37,5 +39,14 @@ int gengetopt_add_option (const char * long_opt, char short_opt,
                           const AcceptedValues *acceptedvalues,
                           int multiple = 0,
                           int argoptional = 0);
+int gengetopt_create_option (gengetopt_option *&opt, const char * long_opt, char short_opt,
+                          const char * desc,
+                          int type, int flagstat, int required,
+                          const char *default_value,
+                          const char * group_value,
+                          const char * type_str,
+                          const AcceptedValues *acceptedvalues,
+                          int multiple = 0,
+                          int argoptional = 0);
 
 #endif /* _GENGETOPT_H */
