@@ -12,6 +12,7 @@ This file is licensed to you under the license specified in the included file
 #endif
 
 #include <list>
+#include <iostream>
 
 #include "acceptedvalues.h"
 
@@ -39,4 +40,6 @@ struct gengetopt_option
 
 typedef std::list<gengetopt_option *> gengetopt_option_list;
 
+std::ostream & operator <<(std::ostream &s, gengetopt_option &opt);
+
 #endif /* _GENGETOPT_GGOS_H */
