@@ -57,14 +57,17 @@ extern "C"
 #include "multiple_fill_array.h"
 #include "multiple_fill_array_default.h"
 #include "free_string.h"
-#include "free_multiple_string.h"
+#include "free_multiple.h"
 #include "free_inputs_string.h"
 #include "reset_group.h"
 #include "exit_failure.h"
 #include "update_given.h"
 #include "given_field.h"
 #include "clear_given.h"
+#include "clear_arg.h"
 #include "free_list.h"
+#include "file_save.h"
+#include "file_save_multiple.h"
 #include "gm_utils.h"
 #include "fileutils.h"
 
@@ -76,6 +79,9 @@ extern "C" char *strdup (const char *s) ;
 
 #define EXE_NAME "argv[0]"
 
+#define PARSER_NAME_PREFIX (c_source_gen_class::parser_name + "_")
+#define OPTION_VALUES_NAME(n) (PARSER_NAME_PREFIX + n + "_values")
+
 using std::endl;
 
 extern char * gengetopt_package;
@@ -179,7 +185,7 @@ CmdlineParserCreator::CmdlineParserCreator (char *function_name,
   if (! gen_strdup)
     {
       foropt
-        if (opt->type == ARG_STRING) {
+        if (opt->type != ARG_FLAG || opt->type != ARG_NO) {
           gen_strdup = true;
           break;
         }
@@ -236,11 +242,13 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
 
   if (opt->arg_is_optional)
     {
-      if (opt->type == ARG_STRING && opt->multiple)
+      if (opt->multiple)
         {
           update_arg_gen_class str_gen_null;
-          str_gen_null.set_string_arg(true);
+
+          str_gen_null.set_string_arg(opt->type == ARG_STRING);
           str_gen_null.set_no_free(true);
+          str_gen_null.set_has_arg(true);
           str_gen_null.set_optarg ("NULL");
           str_gen_null.set_structure (string (opt->var_arg) + "_new");
           str_gen_null.set_opt_var (opt->var_arg);
@@ -254,21 +262,25 @@ CmdlineParserCreator::do_update_arg (struct gengetopt_option *opt,
       indent += offset;
     }
 
+    update_arg_gen.set_has_arg(true);
+
+    if (opt->multiple)
+    {
+      update_arg_gen.set_no_free(true);
+    }
+    else
+    {
+      update_arg_gen.set_no_free(false);
+    }
+
   switch (opt->type)
     {
     case ARG_FLAG:
       update_arg_gen.set_flag_arg(true);
+      update_arg_gen.set_has_arg(false);
       break;
     case ARG_STRING:
       update_arg_gen.set_string_arg(true);
-      if (opt->multiple)
-        {
-          update_arg_gen.set_no_free(true);
-        }
-      else
-        {
-          update_arg_gen.set_no_free(false);
-        }
       break;
     case ARG_INT:
       update_arg_gen.set_int_arg(true);
@@ -380,10 +392,15 @@ _generate_option_arg(ostream &stream,
   option_arg_gen_class option_arg_gen;
 
   string type = arg_types[opt->type];
-  if (opt->multiple)
+  string origtype = "char *";
+
+  if (opt->multiple) {
     type += "*";
+    origtype += "*";
+  }
 
   option_arg_gen.set_type(type);
+  option_arg_gen.set_origtype(origtype);
   option_arg_gen.set_flag_arg((opt->type == ARG_FLAG));
   option_arg_gen.set_desc(opt->desc);
   option_arg_gen.set_name(opt->var_arg);
@@ -472,8 +489,8 @@ CmdlineParserCreator::generate_option_values_decl(ostream &stream,
           first = false;
         }
 
-        stream << "extern char *" << opt->var_arg <<
-          "_values[] ;\t/* Possible values for " << opt->long_opt << ".  */\n";
+        stream << "extern char *" << OPTION_VALUES_NAME(opt->var_arg) <<
+          "[] ;\t/* Possible values for " << opt->long_opt << ".  */\n";
       }
     }
 
@@ -495,8 +512,8 @@ CmdlineParserCreator::generate_option_values(ostream &stream,
           first = false;
         }
 
-        stream << "char *" << opt->var_arg <<
-          "_values[] = {" << opt->acceptedvalues->toString(false) <<
+        stream << "char *" << OPTION_VALUES_NAME(opt->var_arg) <<
+          "[] = {" << opt->acceptedvalues->toString(false) <<
           ", 0} ;\t/* Possible values for " << opt->long_opt << ".  */\n";
       }
     }
@@ -920,22 +937,24 @@ CmdlineParserCreator::free_option(struct gengetopt_option *opt,
   if (! opt->type)
     return;
 
-  if (opt->type == ARG_STRING)
+  if (opt->type != ARG_NO && opt->type != ARG_FLAG)
     {
       if (opt->multiple)
         {
-          free_multiple_string_gen_class free_multiple_string;
-          free_multiple_string.set_comment ("previous argument");
-          free_multiple_string.set_structure (ARGS_STRUCT);
+          free_multiple_gen_class free_multiple;
+          free_multiple.set_comment ("previous argument");
+          free_multiple.set_has_string_type(opt->type == ARG_STRING);
+          free_multiple.set_structure (ARGS_STRUCT);
 
-          free_multiple_string.set_opt_var (opt->var_arg);
-          free_multiple_string.generate_free_multiple_string
+          free_multiple.set_opt_var (opt->var_arg);
+          free_multiple.generate_free_multiple
             (stream, indent);
         }
       else
         {
           free_string_gen_class free_string;
           free_string.set_comment ("previous argument");
+          free_string.set_has_string_type(opt->type == ARG_STRING);
           free_string.set_structure (ARGS_STRUCT);
 
           free_string.set_opt_var (opt->var_arg);
@@ -1069,69 +1088,45 @@ void
 CmdlineParserCreator::generate_clear_arg(ostream &stream, unsigned int indent)
 {
   struct gengetopt_option * opt;
-  string indent_str (indent, ' ');
+  clear_arg_gen_class clear_arg;
 
-  /* now we initialize "given" fields */
+  /* now we initialize value fields */
   foropt
     {
+      if (opt->type == ARG_NO)
+        continue;
+
+      clear_arg.set_name(opt->var_arg);
+      clear_arg.set_suffix("arg");
+      clear_arg.set_value("NULL");
+      clear_arg.set_has_orig(opt->type != ARG_FLAG);
+      clear_arg.set_has_arg(false);
+
       if (opt->multiple && opt->type)
         {
-          // since multiple options are stored in arrays, we
-          // always initialize it to NULL
-
-          stream << indent_str;
-
-          stream << ARGS_STRUCT << "->" << opt->var_arg
-                 << "_arg = NULL;" << endl;
+          clear_arg.set_has_arg(true);
         }
       else if (opt->type == ARG_STRING)
         {
-          stream << indent_str;
-
+          clear_arg.set_has_arg(true);
           if (opt->default_given)
-            stream << ARGS_STRUCT << "->" << opt->var_arg
-                   << "_arg = gengetopt_strdup (\"" << opt->default_string
-                   << "\");";
-          else
-            stream << ARGS_STRUCT << "->" << opt->var_arg
-                   << "_arg = NULL;";
-
-          stream << endl;
+            clear_arg.set_value
+                ("gengetopt_strdup (\"" + string(opt->default_string) +
+                "\")");
         }
       else if (opt->type == ARG_FLAG)
         {
-          stream << indent_str;
-
-          stream << ARGS_STRUCT << "->" << opt->var_arg << "_flag";
-          stream << " = " << opt->flagstat << ";";
-
-          stream << endl;
+          clear_arg.set_has_arg(true);
+          clear_arg.set_suffix("flag");
+          clear_arg.set_value(opt->flagstat ? "1" : "0");
         }
-      else if (opt->type != ARG_NO && opt->default_given)
+      else if (opt->default_given)
         {
-          stream << indent_str;
-
-          stream << ARGS_STRUCT << "->" << opt->var_arg << "_arg";
-          stream << " = " << opt->default_string << " ;";
-
-          stream << endl;
+          clear_arg.set_has_arg(true);
+          clear_arg.set_value(opt->default_string);
         }
-    }
-}
 
-void
-CmdlineParserCreator::generate_init_unamed(ostream &stream,
-                                           unsigned int indent)
-{
-  if ( unamed_options )
-    {
-      string indent_str (indent, ' ');
-
-      stream << endl;
-      stream << indent_str;
-      stream << ARGS_STRUCT << "->inputs = NULL;\n";
-      stream << indent_str;
-      stream << ARGS_STRUCT << "->inputs_num = 0;\n";
+      clear_arg.generate_clear_arg(stream, indent);
     }
 }
 
@@ -1144,9 +1139,6 @@ CmdlineParserCreator::generate_long_option_struct(ostream &stream,
 
   foropt
     {
-      if (opt->short_opt == 'h' || opt->short_opt == 'V')
-        continue;
-
       stream << indent_str;
 
       stream << "{ \"" << opt->long_opt << "\",\t"
@@ -1274,12 +1266,23 @@ CmdlineParserCreator::handle_options(ostream &stream, unsigned int indent, bool
     {
       if ((has_short && opt->short_opt) || (!has_short && !opt->short_opt))
         {
-          if (opt->short_opt == 'h' || opt->short_opt == 'V')
-            continue;
-
           if (has_short || first)
             stream << indent_str;
 
+          if (opt->short_opt == HELP_SHORT_OPT && strcmp(opt->long_opt, HELP_LONG_OPT) == 0) {
+            generate_handle_help(stream, indent);
+            stream << endl;
+            stream << endl;
+            continue;
+          }
+
+          if (opt->short_opt == VERSION_SHORT_OPT && strcmp(opt->long_opt, VERSION_LONG_OPT) == 0) {
+            generate_handle_version(stream, indent);
+            stream << endl;
+            stream << endl;
+            continue;
+          }
+
           string short_opt (1, opt->short_opt);
           if (opt->multiple)
             {
@@ -1291,6 +1294,7 @@ CmdlineParserCreator::handle_options(ostream &stream, unsigned int indent, bool
               multip_opt_gen.set_long_option (opt->long_opt);
               multip_opt_gen.set_option_var_name (opt->var_arg);
               multip_opt_gen.set_option_has_values (opt->acceptedvalues != 0);
+              multip_opt_gen.set_option_values (OPTION_VALUES_NAME(opt->var_arg));
 
               if (opt->type)
                 { // arguments possibly exists
@@ -1323,6 +1327,7 @@ CmdlineParserCreator::handle_options(ostream &stream, unsigned int indent, bool
               option_gen.set_long_option (opt->long_opt);
               option_gen.set_option_var_name (opt->var_arg);
               option_gen.set_option_has_values (opt->acceptedvalues != 0);
+              option_gen.set_option_values (OPTION_VALUES_NAME(opt->var_arg));
 
               ostringstream update_stream;
 
@@ -1386,6 +1391,8 @@ CmdlineParserCreator::generate_handle_group(ostream &stream,
   string indent_str (indent, ' ');
   opt_gen.set_package_var_name (EXE_NAME);
 
+  opt_gen.set_Comparison_rule(GROUP_NOT_REQUIRED_COMPARISON " 1");
+
   groups_collection_t::const_iterator end = gengetopt_groups.end();
   for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
         idx != end; ++idx)
@@ -1395,12 +1402,10 @@ CmdlineParserCreator::generate_handle_group(ostream &stream,
       opt_gen.set_group_var_name (canonize_name (idx->first));
       if (idx->second.required)
         {
-          opt_gen.set_Comparison_rule(GROUP_REQUIRED_COMPARISON);
           opt_gen.set_number_required(GROUP_REQUIRED_MESSAGE);
         }
       else
         {
-          opt_gen.set_Comparison_rule(GROUP_NOT_REQUIRED_COMPARISON);
           opt_gen.set_number_required(GROUP_NOT_REQUIRED_MESSAGE);
         }
 
@@ -1436,6 +1441,28 @@ CmdlineParserCreator::generate_handle_required(ostream &stream,
 
         stream << endl;
       }
+
+  // now generate the checks for required group options
+  group_option_gen_class group_opt_gen;
+  group_opt_gen.set_package_var_name ("prog_name");
+
+  group_opt_gen.set_Comparison_rule("== 0");
+  group_opt_gen.set_number_required(GROUP_REQUIRED_MESSAGE);
+
+  groups_collection_t::const_iterator end = gengetopt_groups.end();
+  for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
+        idx != end; ++idx)
+  {
+    if (idx->second.required)
+    {
+      stream << indent_str;
+      group_opt_gen.set_group_name (idx->first);
+      group_opt_gen.set_group_var_name (canonize_name (idx->first));
+
+      group_opt_gen.generate_group_option (stream, indent);
+      stream << endl;
+    }
+  }
 }
 
 void
@@ -1528,3 +1555,48 @@ CmdlineParserCreator::generate_list_free(ostream &stream,
     }
 }
 
+void
+CmdlineParserCreator::generate_file_save_loop(ostream &stream, unsigned int indent)
+{
+  struct gengetopt_option * opt;
+
+  file_save_multiple_gen_class file_save_multiple;
+  const string suffix = "_orig";
+  const string suffix_given = "_given";
+
+  foropt {
+    file_save_gen_class file_save;
+    file_save.set_opt_name(opt->long_opt);
+
+    if (opt->multiple) {
+      file_save.set_check_given(false);
+    } else {
+      file_save.set_check_given(true);
+      file_save.set_given(opt->var_arg + suffix_given);
+    }
+
+    if (opt->type != ARG_NO && opt->type != ARG_FLAG) {
+      file_save.set_arg(opt->var_arg + suffix + (opt->multiple ? " [i]" : ""));
+      file_save.set_has_arg(true);
+    } else {
+      file_save.set_has_arg(false);
+    }
+
+    if (opt->multiple) {
+      ostringstream buffer;
+
+      file_save.generate_file_save(buffer, indent);
+
+      file_save_multiple.set_has_arg(opt->type != ARG_NO);
+      file_save_multiple.set_opt_var(opt->var_arg);
+      file_save_multiple.set_opt_name(opt->long_opt);
+      file_save_multiple.set_write_cmd(buffer.str());
+
+      file_save_multiple.generate_file_save_multiple(stream, indent);
+    } else {
+      file_save.generate_file_save(stream, indent);
+    }
+  }
+}
+
+
