@@ -14,6 +14,9 @@
 #include "my_sstream.h"
 #include "ggo_options.h"
 #include "argsdef.h"
+#include "groups.h"
+
+extern groups_collection_t gengetopt_groups;
 
 /*
   return a copy of the string passed after canonizing it (i.e. '-' and
@@ -107,13 +110,13 @@ has_multiple_options()
 bool
 has_multiple_options_with_type()
 {
-  struct gengetopt_option * opt = 0;
+  gengetopt_option * opt = 0;
 
-  foropt
-  {
+  for (gengetopt_option_list::iterator it = gengetopt_options.begin();
+                    it != gengetopt_options.end() && (opt = *it);
+                    ++it)
     if (opt->multiple && opt->type)
       return true;
-  }
 
   return false;
 }
@@ -129,6 +132,16 @@ has_required()
       return true;
   }
 
+  groups_collection_t::const_iterator end = gengetopt_groups.end();
+  for ( groups_collection_t::const_iterator idx = gengetopt_groups.begin();
+        idx != end; ++idx)
+  {
+    if (idx->second.required)
+    {
+      return true;
+    }
+  }
+
   return false;
 }
 
@@ -154,8 +167,10 @@ has_values()
 {
   struct gengetopt_option * opt = 0;
 
-  foropt
-  {
+  for (gengetopt_option_list::iterator it = gengetopt_options.begin();
+       it != gengetopt_options.end();
+       ++it) {
+    opt = *it;
     if (opt->acceptedvalues)
       {
         return true;
@@ -207,8 +222,8 @@ wrap_cstr( string& wrapped, unsigned int from_column, unsigned int second_indent
       if (*out_buf)
         if ((newline_chars = char_is_newline(out_buf, num_of_newlines)))
           {
-	    for (int i = 1; i <= num_of_newlines; ++i)
-	      stream << "\\n";
+            for (int i = 1; i <= num_of_newlines; ++i)
+              stream << "\\n";
 
             out_buf += newline_chars;
 
@@ -231,7 +246,7 @@ wrap_cstr( string& wrapped, unsigned int from_column, unsigned int second_indent
       while ((*out_buf) && (*out_buf != ' ') &&
              ! char_is_newline(out_buf, num_of_newlines))
         {
-	  // num_of_newlines is not used.
+          // num_of_newlines is not used.
           next_word += *out_buf++;
           next_space++;
         }
