@@ -24,7 +24,10 @@ const string to_upper(const string &);
 
 bool has_multiple_options_string();
 bool has_multiple_options();
+bool has_multiple_options_with_type();
+bool has_required();
 bool has_options();
+bool has_values();
 
 void wrap_cstr (string &wrapped, unsigned int from_column, unsigned int second_indent, const string &orig);
 
