@@ -113,6 +113,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include "my_sstream.h"
 
 #include "acceptedvalues.h"
 
@@ -134,25 +135,36 @@ extern int yylex () ;
 
 //#define YYERROR_VERBOSE 1
 
-#define check_result					\
-    if (o) {						\
-	switch (o) {					\
-	    case NOT_ENOUGH_MEMORY: yyerror2 ("not enough memory");	\
-		break;					\
-	    case REQ_LONG_OPTION: yyerror2 ("long option redefined");	\
-		break;					\
-	    case REQ_SHORT_OPTION: yyerror2 ("short option redefined");	\
-		break;					\
-	    case FOUND_BUG: yyerror2 ("bug found!!");		\
-		break;					\
-	    case GROUP_UNDEFINED: yyerror2 ("group undefined");	\
-		break;					\
-	    case INVALID_DEFAULT_VALUE: yyerror2 ("invalid default value");	\
-		break;					\
-            case NOT_REQUESTED_TYPE: yyerror2 ("type specification not requested");	\
-		break;					\
-	} YYERROR;					\
+void check_result(int o, const char *long_opt, char shor_opt)
+{
+    if (o) {
+        ostringstream err;
+
+	switch (o) {
+	    case NOT_ENOUGH_MEMORY: yyerror2 ("not enough memory");
+		break;
+	    case REQ_LONG_OPTION:
+                err << "long option redefined \'" << long_opt << "\'";
+	        yyerror2 (err.str().c_str());
+		break;
+	    case REQ_SHORT_OPTION:
+                err << "short option redefined \'" << shor_opt << "\'";
+	        yyerror2 (err.str().c_str());
+		break;
+	    case FOUND_BUG: yyerror2 ("bug found!!");
+		break;
+	    case GROUP_UNDEFINED: yyerror2 ("group undefined");
+		break;
+	    case INVALID_DEFAULT_VALUE: yyerror2 ("invalid default value");
+		break;
+            case NOT_REQUESTED_TYPE: yyerror2 ("type specification not requested");
+		break;
+	}
     }
+}
+
+#define check_error if (o) YYERROR;
+
 
 
 /* Enabling traces.  */
@@ -169,7 +181,7 @@ extern int yylex () ;
 #endif
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 68 "parser.yy"
+#line 80 "parser.yy"
 typedef union YYSTYPE {
     char   *str;
     char    chr;
@@ -178,7 +190,7 @@ typedef union YYSTYPE {
     class AcceptedValues *ValueList;
 } YYSTYPE;
 /* Line 191 of yacc.c.  */
-#line 182 "parser.cc"
+#line 194 "parser.cc"
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
@@ -190,7 +202,7 @@ typedef union YYSTYPE {
 
 
 /* Line 214 of yacc.c.  */
-#line 194 "parser.cc"
+#line 206 "parser.cc"
 
 #if ! defined (yyoverflow) || YYERROR_VERBOSE
 
@@ -381,11 +393,11 @@ static const yysigned_char yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const unsigned short int yyrline[] =
 {
-       0,   119,   119,   121,   126,   127,   128,   129,   130,   131,
-     132,   137,   157,   177,   197,   204,   215,   220,   229,   241,
-     242,   246,   247,   251,   252,   253,   257,   258,   262,   263,
-     267,   268,   272,   273,   277,   278,   282,   283,   287,   288,
-     292,   293,   297,   298,   302,   303,   307
+       0,   131,   131,   133,   138,   139,   140,   141,   142,   143,
+     144,   149,   169,   189,   209,   216,   227,   233,   243,   256,
+     257,   261,   262,   266,   267,   268,   272,   273,   277,   278,
+     282,   283,   287,   288,   292,   293,   297,   298,   302,   303,
+     307,   308,   312,   313,   317,   318,   322
 };
 #endif
 
@@ -1138,7 +1150,7 @@ yyreduce:
   switch (yyn)
     {
         case 11:
-#line 138 "parser.yy"
+#line 150 "parser.yy"
     {
 	      if (gengetopt_package_given)
 		{
@@ -1158,7 +1170,7 @@ yyreduce:
     break;
 
   case 12:
-#line 158 "parser.yy"
+#line 170 "parser.yy"
     {
 	      if (gengetopt_version_given)
 		{
@@ -1178,7 +1190,7 @@ yyreduce:
     break;
 
   case 13:
-#line 178 "parser.yy"
+#line 190 "parser.yy"
     {
 	      if (gengetopt_purpose_given)
 		{
@@ -1198,14 +1210,14 @@ yyreduce:
     break;
 
   case 14:
-#line 198 "parser.yy"
+#line 210 "parser.yy"
     {
                 gengetopt_set_section (yyvsp[-1].str, yyvsp[0].str);
               }
     break;
 
   case 15:
-#line 205 "parser.yy"
+#line 217 "parser.yy"
     {
 	      if (gengetopt_add_group (yyvsp[-2].str, yyvsp[-1].str, yyvsp[0].boolean))
 		{
@@ -1216,156 +1228,159 @@ yyreduce:
     break;
 
   case 16:
-#line 216 "parser.yy"
+#line 228 "parser.yy"
     {
 	      int o = gengetopt_add_option (yyvsp[-4].str, yyvsp[-3].chr, yyvsp[-2].str, ARG_FLAG, yyvsp[0].boolean, 0, 0, 0, 0, 0);
-	      check_result;
+	      check_result(o, yyvsp[-4].str, yyvsp[-3].chr);
+	      check_error;
 	    }
     break;
 
   case 17:
-#line 222 "parser.yy"
+#line 235 "parser.yy"
     {
 	      int o = gengetopt_add_option (yyvsp[-9].str, yyvsp[-8].chr, yyvsp[-7].str, yyvsp[-6].argtype, 0, yyvsp[-2].boolean, yyvsp[-3].str, 0, yyvsp[-5].str, yyvsp[-4].ValueList, yyvsp[0].boolean, yyvsp[-1].boolean);
-	      check_result;
+	      check_result(o, yyvsp[-9].str, yyvsp[-8].chr);
+	      check_error;
 	    }
     break;
 
   case 18:
-#line 231 "parser.yy"
+#line 245 "parser.yy"
     {
 	      int o = gengetopt_add_option (yyvsp[-9].str, yyvsp[-8].chr, yyvsp[-7].str, yyvsp[-6].argtype, 0, 0, yyvsp[-3].str, yyvsp[-2].str, yyvsp[-5].str, yyvsp[-4].ValueList, yyvsp[0].boolean, yyvsp[-1].boolean);
-	      check_result;
+	      check_result(o, yyvsp[-9].str, yyvsp[-8].chr);
+	      check_error;
 	    }
     break;
 
   case 21:
-#line 246 "parser.yy"
+#line 261 "parser.yy"
     { yyval.boolean = 1; }
     break;
 
   case 22:
-#line 247 "parser.yy"
-    { yyval.boolean = 0; }
-    break;
-
-  case 23:
-#line 251 "parser.yy"
-    { yyval.boolean = 0; }
-    break;
-
-  case 24:
-#line 252 "parser.yy"
-    { yyval.boolean = 1; }
-    break;
-
-  case 25:
-#line 253 "parser.yy"
-    { yyval.boolean = 0; }
-    break;
-
-  case 26:
-#line 257 "parser.yy"
-    { yyval.boolean = 0; }
-    break;
-
-  case 27:
-#line 258 "parser.yy"
-    { yyval.boolean = 1; }
-    break;
-
-  case 28:
 #line 262 "parser.yy"
     { yyval.boolean = 0; }
     break;
 
+  case 23:
+#line 266 "parser.yy"
+    { yyval.boolean = 0; }
+    break;
+
+  case 24:
+#line 267 "parser.yy"
+    { yyval.boolean = 1; }
+    break;
+
+  case 25:
+#line 268 "parser.yy"
+    { yyval.boolean = 0; }
+    break;
+
+  case 26:
+#line 272 "parser.yy"
+    { yyval.boolean = 0; }
+    break;
+
+  case 27:
+#line 273 "parser.yy"
+    { yyval.boolean = 1; }
+    break;
+
+  case 28:
+#line 277 "parser.yy"
+    { yyval.boolean = 0; }
+    break;
+
   case 29:
-#line 263 "parser.yy"
+#line 278 "parser.yy"
     { yyval.boolean = 1; }
     break;
 
   case 30:
-#line 267 "parser.yy"
+#line 282 "parser.yy"
     { yyval.argtype = ARG_NO; }
     break;
 
   case 31:
-#line 268 "parser.yy"
+#line 283 "parser.yy"
     { yyval.argtype = yyvsp[0].argtype; }
     break;
 
   case 32:
-#line 272 "parser.yy"
-    { yyval.str = 0; }
-    break;
-
-  case 33:
-#line 273 "parser.yy"
-    { yyval.str = yyvsp[0].str; }
-    break;
-
-  case 34:
-#line 277 "parser.yy"
-    { yyval.str = 0; }
-    break;
-
-  case 35:
-#line 278 "parser.yy"
-    { yyval.str = yyvsp[0].str; }
-    break;
-
-  case 36:
-#line 282 "parser.yy"
-    { yyval.str = 0; }
-    break;
-
-  case 37:
-#line 283 "parser.yy"
-    { yyval.str = yyvsp[0].str; }
-    break;
-
-  case 38:
 #line 287 "parser.yy"
     { yyval.str = 0; }
     break;
 
-  case 39:
+  case 33:
 #line 288 "parser.yy"
     { yyval.str = yyvsp[0].str; }
     break;
 
-  case 40:
+  case 34:
 #line 292 "parser.yy"
     { yyval.str = 0; }
     break;
 
-  case 41:
+  case 35:
 #line 293 "parser.yy"
     { yyval.str = yyvsp[0].str; }
     break;
 
-  case 42:
+  case 36:
 #line 297 "parser.yy"
+    { yyval.str = 0; }
+    break;
+
+  case 37:
+#line 298 "parser.yy"
+    { yyval.str = yyvsp[0].str; }
+    break;
+
+  case 38:
+#line 302 "parser.yy"
+    { yyval.str = 0; }
+    break;
+
+  case 39:
+#line 303 "parser.yy"
+    { yyval.str = yyvsp[0].str; }
+    break;
+
+  case 40:
+#line 307 "parser.yy"
+    { yyval.str = 0; }
+    break;
+
+  case 41:
+#line 308 "parser.yy"
+    { yyval.str = yyvsp[0].str; }
+    break;
+
+  case 42:
+#line 312 "parser.yy"
     { yyval.ValueList = 0; }
     break;
 
   case 43:
-#line 298 "parser.yy"
+#line 313 "parser.yy"
     { yyval.ValueList = yyvsp[0].ValueList; }
     break;
 
   case 44:
-#line 302 "parser.yy"
+#line 317 "parser.yy"
     { yyval.ValueList = new AcceptedValues; yyval.ValueList->insert(yyvsp[0].str); }
     break;
 
   case 45:
-#line 303 "parser.yy"
+#line 318 "parser.yy"
     { yyvsp[-2].ValueList->insert(yyvsp[0].str); yyval.ValueList = yyvsp[-2].ValueList; }
     break;
 
   case 46:
-#line 307 "parser.yy"
+#line 322 "parser.yy"
     { yyval.str = yyvsp[0].str; }
     break;
 
@@ -1373,7 +1388,7 @@ yyreduce:
     }
 
 /* Line 1010 of yacc.c.  */
-#line 1377 "parser.cc"
+#line 1392 "parser.cc"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1598,6 +1613,6 @@ yyreturn:
 }
 
 
-#line 310 "parser.yy"
+#line 325 "parser.yy"
 
 
